'use strict';

var mod = require('../../src/data-structures/linked-list.js');
var Node = mod.Node;
var LinkedList = mod.LinkedList;

describe('Node', function () {
  it('should be a constructor function', function () {
    expect(typeof Node).toBe('function');
  });
  it('should construct properly', function () {
    var node = new Node("data");
    expect(node.data).toBe("data");
    expect(node.next).toBe(null);
    expect(node.prev).toBe(null);
  });
});

describe('Linked List', function () {
  it('should be a constructor function', function () {
    expect(typeof LinkedList).toBe('function');
  });
  it('should push properly', function () {
    var linkedList = new LinkedList();
    linkedList.push(1);
    linkedList.push(2);
    linkedList.push(3);
    linkedList.push(4);
    linkedList.push(5);
    expect(linkedList.first.data).toBe(1);
    expect(linkedList.first.next.data).toBe(2);
    expect(linkedList.first.next.next.data).toBe(3);
    expect(linkedList.first.next.next.next.data).toBe(4);
    expect(linkedList.first.next.next.next.next.data).toBe(5);
    expect(linkedList.last.data).toBe(5);
  });
  it('should pop properly', function () {
    var linkedList = new LinkedList();
    linkedList.push(1);
    linkedList.push(2);
    linkedList.push(3);
    linkedList.push(4);
    linkedList.push(5);
    expect(linkedList.pop().data).toBe(5);
    expect(linkedList.pop().data).toBe(4);
    expect(linkedList.pop().data).toBe(3);
    expect(linkedList.pop().data).toBe(2);
    expect(linkedList.pop().data).toBe(1);
  });
  it('should shift properly', function () {
    var linkedList = new LinkedList();
    linkedList.push(1);
    linkedList.push(2);
    linkedList.push(3);
    linkedList.push(4);
    linkedList.push(5);
    expect(linkedList.shift().data).toBe(1);
    expect(linkedList.shift().data).toBe(2);
    expect(linkedList.shift().data).toBe(3);
    expect(linkedList.shift().data).toBe(4);
    expect(linkedList.shift().data).toBe(5);
  });
  it('should reverse properly', function () {
    var linkedList = new LinkedList();
    linkedList.push(1);
    linkedList.push(2);
    linkedList.push(3);
    linkedList.push(4);
    linkedList.push(5);
    linkedList.reverse();
    expect(linkedList.shift().data).toBe(5);
    expect(linkedList.shift().data).toBe(4);
    expect(linkedList.shift().data).toBe(3);
    expect(linkedList.shift().data).toBe(2);
    expect(linkedList.shift().data).toBe(1);
  });
  it('should recursive reverse properly', function () {
    var linkedList = new LinkedList();
    linkedList.push(1);
    linkedList.push(2);
    linkedList.push(3);
    linkedList.push(4);
    linkedList.push(5);
    linkedList.recursiveReverse();
    expect(linkedList.shift().data).toBe(5);
    expect(linkedList.shift().data).toBe(4);
    expect(linkedList.shift().data).toBe(3);
    expect(linkedList.shift().data).toBe(2);
    expect(linkedList.shift().data).toBe(1);
  });
  it('should unshift properly', function () {
    var linkedList = new LinkedList();
    linkedList.push(1);
    linkedList.push(2);
    linkedList.push(3);
    linkedList.push(4);
    linkedList.push(5);
    linkedList.unshift(3);
    expect(linkedList.shift().data).toBe(3);
    expect(linkedList.shift().data).toBe(1);
    expect(linkedList.shift().data).toBe(2);
    expect(linkedList.shift().data).toBe(3);
    expect(linkedList.shift().data).toBe(4);
    expect(linkedList.shift().data).toBe(5);
  });
  it('should properly check for existing cycle', function () {
    var linkedList = new LinkedList();
    var last = new Node(2);
    var first = new Node(1);
    last.next = first;
    last.prev = first;
    first.next = last;
    first.prev = last;
    linkedList.first = first;
    linkedList.last = last;
    expect(linkedList.hasCycle()).toBe(true);
  });
  it('should properly check for non existing cycle', function () {
    var linkedList = new LinkedList();
    linkedList.push(1);
    linkedList.push(2);
    linkedList.push(3);
    linkedList.push(4);
    linkedList.push(5);
    expect(linkedList.hasCycle()).toBe(false);
  });
  it('should inorder properly', function () {
    var linkedList = new LinkedList();
    linkedList.push(1);
    linkedList.push(2);
    linkedList.push(3);
    linkedList.push(4);
    linkedList.push(5);
    var pushedValue = 1;
    function callback(node){
      expect(node.data).toBe(pushedValue++);
    }
    linkedList.inorder(callback);
  });
});
